/* 
 * aero.h - common definitions for aerodynamic computations
 *
 */

#include <math.h>
#include "falcon/lutable.h"

#define DEG2RAD      (M_PI/180.0)
#define RAD2DEG      (180.0/M_PI)
#define RHO 1.2247			/* density of air (sealevel), kg/m^3 */
#define CHORD 0.3556            /* wing chord */
/* #define SPAN 0.6604 */            /* wing span */ 
#define SPAN 0.50             /* wing span */
/*#define AREA_FAN 1 */         /* frontal area */
#define FAN_DIST 2.032          /* dist to center of shroud */
#define BOOM_Cd 1.2             /* 1.0 according to Schaum */
#define BOOM_DIA 0.0635         /* diameter */
#define BOOM_CEN_DIST 0.8763    /* dist to midpoint of boom */
#define WING_INSIDE_DIST (2.15)  /* dist to inside edge of wing */
/* #define WING_OUTSIDE_DIST (2.81) */  /* dist to outside edge of wing */ 
#define WING_OUTSIDE_DIST (2.65)   /* dist to outside edge of wing */
#define WING_CEN_DIST (0.5*(WING_INSIDE_DIST + WING_OUTSIDE_DIST))
/* dist to center of wing */
#define WING_SENSOR_DIST   (0.137) 
/* distance between leading wing edge and sensor mount */
#define LE_CM_DIST   (0.05)  /* 0.07 */
/* distance between leading wing edge and total fan c.m. 
* Note LE is in front of CM 
*/
#define CM_SHROUDEDGE_DIST   (0.13)  /* 0.12 */
/* distance CM to shroud end edge where forces act
*/

/* mass properties: */
#define mx    (4.9)   /* effective inertial mass in x-direction in kg */
#define mz    (8.36)   /* effective inertial mass in z-direction in kg */
#define mg    (0.46)   /* effective heavy mass in z-direction in kg */
#define grav  (9.81)    /* gravitational constant in kg m/s^2 */
#define Jtheta (0.0323)  /* inertia of fan about pitch axis in kg m^2/rad */
#define Jm (0.166/1000)         /* inertia of motor + blade kg m^2 */
#define Kf 71.42               
/* Cobalt 40 motor constant: 682 V/rpm = 682/60*2*pi (rad/sec) */
#define f_stiction (0.2)  /* stiction in [N] */

#define ALPHA_STALL_MAX  (0.26)  /* +15deg for NACA 0015 */
#define ALPHA_STALL_MIN  (-0.26)  /* -15deg for NACA 0015 */

#define DELTA_E_MAX  (1.00)  /* 1 rad = +60deg */
#define DELTA_E_MIN  (-1.00)  /* -1 rad = -60deg */

/* indices; */
#define LIFT 0
#define DRAG 1
#define MOMENT 2

#define FX 0
#define FZ 1


/* thrust limits from static thrust table: */
#define FPARAMAX   (16.0)
#define FPARAMIN   (0.0)
#define FPERPMAX   (6.25)
#define FPERPMIN   (-6.25)


/* Function declarations */
extern char *dfspec(char *, char *);
extern int init_wing(char *table);
extern int compute_wingLDM(double Vel, double alpharad, double delta_e_rad, 
			   double *wingLDM);

/* Global variable declarations */
extern int aero_verbose;

/* Table declarations */
extern LU_DATA *shroud_table;
extern LU_DATA *wing_table;

#define WING_LOOKUP_TABLE	"milam-may96/wing_forward.lut"
#define SHROUD_LOOKUP_TABLE	"milam-may96/shroud_forward.lut"
#define FORCE_TABLE		"milam-may96/shroud_inverse.lut"
