#include "aero.h"

static double wingscalex, boomscalex, wingscalez, boomscalez;		


int init_aero(int flag)
{
    double tmp0, tmp1;

    if (init_wing(WING_LOOKUP_TABLE) < 0) return -1;
    if (init_shroud(SHROUD_LOOKUP_TABLE) < 0) return -1;
    if (init_boom() < 0) return -1;

    /* initialize scaling constants: */
    tmp0 = WING_INSIDE_DIST; 
    tmp1 = WING_OUTSIDE_DIST; 
    wingscalex = (tmp1*tmp1*tmp1*tmp1 - tmp0*tmp0*tmp0*tmp0)/(4.0*(tmp1-tmp0)*FAN_DIST*FAN_DIST*FAN_DIST);
    wingscalez = (tmp1*tmp1*tmp1 - tmp0*tmp0*tmp0)/(3.0*(tmp1-tmp0)*FAN_DIST*FAN_DIST);
    
    /* Compute corrective factor for boom rotation */
    tmp0 = 0;
    tmp1 = 2*BOOM_CEN_DIST; 
    boomscalex = (tmp1*tmp1*tmp1*tmp1 - tmp0*tmp0*tmp0*tmp0)/(4.0*(tmp1-tmp0)*FAN_DIST*FAN_DIST*FAN_DIST);
    boomscalez = (tmp1*tmp1*tmp1 - tmp0*tmp0*tmp0)/(3.0*(tmp1-tmp0)*FAN_DIST*FAN_DIST);

    if(aero_verbose>5) {
	printf("boomscale[x,y] = %f; %f\n", boomscalex, boomscalez);
	printf("wingscale[x,y] = %f; %f\n", wingscalex, wingscalez);
    }

    aero_verbose = flag;

    return flag;
}

/***************************************************************/
int voltangle_to_force(double Voltm, double paddlerad, double delta_e_rad, double thetarad, 
		       double xdot0, double zdot, double *Fspatial)
/***************************************************************/
/* 
 * Note: this returns total force in Fspatial. Not just aero forces. 
 * 
 * Voltm: Voltmage to motor [V]
 * paddle angle [rad]
 * delta_e: elevator deflection [rad]
 * theta: pitch angle [rad]
 * xdot: in spatial coordinates [m/s]
 * zdot: in spatial coordinates [m/s]
 */
{
    static double forces0[3][3], forces1[3][3], fspat_anl[3]; 
    int verbose = 1, j, i;
    double alphaVgam[3], alpharad, Vel, gammarad, cgamma, sgamma;
    
    compute_alphaVgamma(thetarad, xdot0, zdot, alphaVgam);
    alpharad = alphaVgam[0]; 
    Vel      = alphaVgam[1];
    gammarad = alphaVgam[2];

    
#ifdef AERO_VERBOSE
    if(verbose > 0) 
	printf("\nVel = %5.4f [m/s], alpha = %5.4f [deg], gamma = %5.4f [deg]\n", 
	       Vel, alpharad*RAD2DEG, gammarad*RAD2DEG);
#endif
    
    compute_boomLDM(     Vel, forces0[0]);
    compute_shroudLDM(   Vel, alpharad, paddlerad, Voltm, forces0[1]);
    compute_wingLDM(     Vel, alpharad, delta_e_rad, forces0[2]);
    if (verbose > 2) printf("done computing forces\n");
    
    /* transform boom,shroud,wing LDM to spatial forces (see documentation.ps). 
     */
    cgamma = cos(gammarad);
    sgamma = sin(gammarad);
    for(j=0; j<=2; j++){
	forces1[j][FX] =  - sgamma*forces0[j][LIFT] -cgamma*forces0[j][DRAG];
	forces1[j][FZ] =  - cgamma*forces0[j][LIFT] +sgamma*forces0[j][DRAG];
	forces1[j][MOMENT] = forces0[j][MOMENT];
    }
    if (verbose > 2) printf("done transforming forces\n");
    
    
#ifndef PLANAR
    /* Transform boom and wingforces in x-direction 
     * to frame centered at fan shroud. 
     * This is because the fan moves on a cylinder : 
     */
    Fspatial[FX] = forces1[0][FX]*boomscalex + forces1[1][FX] + forces1[2][FX]*wingscalex;
    Fspatial[FZ] = forces1[0][FZ]*boomscalez + forces1[1][FZ] + forces1[2][FZ]*wingscalez;
    Fspatial[MOMENT] = forces1[0][2] + forces1[1][2] + forces1[2][2];
#else
    /* in planar case, just sum: */
    for(i=0; i<3; i++){
	Fspatial[i] = 0.0;
	for(j=0; j<3; j++) {
	    Fspatial[i] += forces1[j][i]; 
	}
    }
#endif
    
    if(xdot0<0.0) {
#ifdef AERO_VERBOSE
	printf("rotating back to 2-3 quadrant\n");
#endif
	Fspatial[0] = -Fspatial[0];  /* flip fx */
	Fspatial[1] = -Fspatial[1];  /* flip fz */
	Fspatial[2] = Fspatial[2];  /* flip M */
    }

#ifdef AERO_VERBOSE
    if (verbose > 0) {
	printf(" = total Fspatial [Fx, Fz, M] due to aero+fan: ");
	for(i=0; i<3; i++) printf("%4.6f ", Fspatial[i]);
	printf("\n");
    }
#endif
    
    if (verbose > 1) printf("leaving Voltangle_to_force()\n");
    return 1; 
} /*** end voltangle_to_force() ***/



