/*
 * fansim.c - S-Function for white Milam fan on stand. 
 *
 * MvN 3/24/96, 
 * modified FB 5/13/96
 *
 * Inputs:	volt, paddle[rad], elevator[rad] 
 * Outputs:	x, z, theta [rad], xdot, zdot, thetadot   ... (full state in spatial coords)
 * Inputargs:   none
 *
 * Comments: relies on routines in aerofun.[ch] for most computations.
 *
 * See: documentation.ps for details on names, order and units of variables!! 
 *
 * Directives: PLANAR, STICTION, BLADEROT, NOXINIT
 *
 */

#define VERBOSE 0			/* verbosity level */
#define S_FUNCTION_NAME fansim
#include "simstruc.h"
#include <math.h>
#include "aero.h"

#undef STICTION
#undef BLADEROT
#undef PLANAR

#define Arg0 ssGetArg(S,0)
#define max(a,b)  ((a>b)?a:b)
#define min(a,b)  ((a<b)?a:b)

/* mdlInitializeSizes - initialize the sizes array */
static void mdlInitializeSizes(S)
SimStruct *S;
{
    double xcm,rcm;
    
    ssSetNumContStates(    S, 6);      /* number of continuous states */
    ssSetNumDiscStates(    S, 0);      /* number of discrete states */
    ssSetNumInputs(        S, 3);      /* number of inputs: fperp, fpara, delta_e */
    ssSetNumOutputs(       S, 6);      /* number of outputs */
    ssSetDirectFeedThrough(S, 0);      /* direct feedthrough flag */
    ssSetNumSampleTimes(   S, 1);      /* number of sample times */
#ifndef NOXINIT
    ssSetNumInputArgs(     S, 1);      /* number of input arguments (parms) */
#else
    ssSetNumInputArgs(     S, 0);      /* number of input arguments (parms) */
#endif
    ssSetNumRWork(         S, 0);      /* number real work vector elements */
    ssSetNumIWork(         S, 0);      /* number int work vector elements */
    ssSetNumPWork(         S, 0);      /* number ptr work vector elements */

}

/* mdlInitializeSampleTimes - initialize the sample times array */
static void mdlInitializeSampleTimes(S)
SimStruct *S;
{
    static int firsttime = 1;
    /* Set things up to run as continuous time block */
    ssSetSampleTimeEvent(S, 0, 0.0);
    ssSetOffsetTimeEvent(S, 0, 0.0);

    if(firsttime == 0) return; 
    firsttime = 0;

    if (init_aero(VERBOSE) < 0) {
	fprintf(stderr, "fansim: can't initialize aero database\n");
	return;
    }
}

/* mdlInitializeConditions - initialize the states */
static void mdlInitializeConditions(x0, S)
double *x0;
SimStruct *S;
{
    int i;
    double *arg0;
    static int firsttime = 1;
    
    /* Initialize the states */

#ifndef NOXINIT
    arg0 = mxGetPr(Arg0);
    for (i = 0; i < 6; i++) x0[i] = *(arg0+i);
#endif

    if(aero_verbose>1)
	for(i=0; i<6; i++) printf("x0[%d] = %lf\n", i, x0[i]);

    if(firsttime == 1) {
	firsttime = 0;
    }
}

/* mdlOutputs - compute the outputs */
static void mdlOutputs(y, x, u, S, tid)
double *y, *x, *u;
SimStruct *S;
int tid;
{
    int i;
    for (i = 0; i < 6; i++) y[i] = x[i];
}

/* mdlUpdate - perform action at major integration time step (not used) */
static void mdlUpdate(x, u, S, tid)
double *x, *u;
SimStruct *S;
int tid;
{
    /* Not used for this model */
}

/* mdlDerivatives - compute the derivatives */
static void mdlDerivatives(dx, x, u, S, tid)
double *dx, *x, *u;
SimStruct *S;
int tid;
{
    /* Fspatial = [Fxspatial, Fzspatial, M] */
    double Fspatial[3], theta = x[2], xdot = x[3], zdot = x[4], thetadot = x[5];
    int i;

    /* compute aero forces */
    voltangle_to_force(u[0], u[1], u[2], theta, xdot, zdot, Fspatial);
    if(aero_verbose > 2) printf("returned from voltangle_to_force()\n");
    if(aero_verbose > 10) {
	fprintf(stderr, "time: %f\n", S->T);
	fprintf(stderr, "u: ");
	for(i=0; i<3; i++) fprintf(stderr, "%f ", u[i]);
	fprintf(stderr, "\n");
	fprintf(stderr, "Fspatial: ");
	for(i=0; i<3; i++) fprintf(stderr, "%f ", Fspatial[i]);
	fprintf(stderr, "\n");
    }
    
    /* add gravity: mg is "gravity mass" */
    Fspatial[1] += mg*grav;
    
    
#   ifdef BLADEROT
    /* Include effects due to rotation of fan blade */
    Fspatial[2] += Jm*(u[0]*Kf)*sin(theta)*(xdot/FAN_DIST);
    /* check the sign: this is a pusher */
#   endif
#   ifdef STICTION
    /* only in y-direction : */
    if(zdot>0.0) Fspatial[1] -= f_stiction;
    if(zdot<0.0) Fspatial[1] += f_stiction;
    if(zdot==0.0){
	if(Fspatial[1]>0.0) Fspatial[1] = max(0.0, Fspatial[1] - f_stiction);
	if(Fspatial[1]<0.0) Fspatial[1] = min(0.0, Fspatial[1] + f_stiction);
    }
#   endif
    
    /* give outputs */
    dx[0] = xdot; 
    dx[1] = zdot; 
    dx[2] = thetadot; 
    dx[3] = Fspatial[0]/mx; 
    dx[4] = Fspatial[1]/mz; 
    dx[5] = Fspatial[2]/Jtheta; 
    
    
    if(aero_verbose>2){
	for(i=0; i<6; i++) fprintf(stderr, "x[%d] = %4.2e ", i, x[i]);
	fprintf(stderr, "\n");
	for(i=0; i<6; i++) fprintf(stderr, "dx[%d] = %4.2e ", i, dx[i]);
	fprintf(stderr, "\n");
    }
}

/* mdlTerminate - called when the simulation is terminated */
static void mdlTerminate(S)
SimStruct *S;
{
}

/* Trailer information to set everything up for simulink usage */
#ifdef MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#endif
