/* 
 * shroud.c - aerodynamic computations for shroud
 *
 * MvN 3/24/96 (orig).  
 * FB, 5/13/96, changed notations and added new shroud forward table
 *		(shroudmethod = 5). For conventions on variables see
 *		documentation.ps 
 * RMM, 5/19/96 split off shroud functions into separate file
 *
 */

#include <stdio.h>
#include "aero.h"			/* common declarations */

/* Global variables */
LU_DATA *shroud_table = NULL;

/* Initialize shroud tables */
int init_shroud(char *table)
{
    char filename[1024];

    if (table == NULL) table = SHROUD_LOOKUP_TABLE;
    
    if (aero_verbose > 0)
	fprintf(stderr, "shroud_method = 5: table = %s\n", table);
    
    /* Look for the table in all the usual places */
    if (dfspec(table, filename) == NULL ||
	(shroud_table = lu_load(filename)) == NULL) {
	printf("shround: error Loading %s", table);
	return -1;
    }
    if (aero_verbose > 0) lu_dispinfo(shroud_table);
    return 1; 
}

/*
 * Compute shroud forces based on current state 
 *
 * Tables stores forces and moment directly. 
 * Drag is along the negative wind x-axis. 
 *
 */
int compute_shroudLDM(double Vel, double alpharad, double paddlerad,
		      double Voltm, double *shroudLDM)
{
    register int i;
    double shroud_in[4], shroud_out[3];

    /* table "shroud_forward.lut" is:
     * [Vel, Vm, alpha_deg, deltap_deg] --> [Fxbody, Fzbody, Moment] units=(N,N,Nmm)
     */    
    shroud_in[0] = Vel;
    shroud_in[1] = Voltm;
    shroud_in[2] = alpharad*RAD2DEG;
    shroud_in[3] = paddlerad*RAD2DEG;
        
    if(aero_verbose > 2) {
	printf("Variables into shroud table: (Voltm, Vel, alpha[deg], paddle[deg]): \n");
	for(i=0; i<4; i++) printf("%4.6f ", shroud_in[i]);
	printf("\n");
    }
    if (fabs(alpharad * RAD2DEG) > (50.0))
	if (aero_verbose > 1) printf(" ! Forward shroud table out of range (|alpha|>50deg)\n");
	
    lu_lookup(shroud_table, shroud_in, shroud_out);
    /* shroud_out = [Fxbody, Fxbody, M]. */	

    if(aero_verbose > 2) {
	printf("forward shroud table ins and outs: \n");
	for(i=0; i<4; i++) printf("%4.6f ", shroud_in[i]); 
	printf("\n");
	for(i=0; i<3; i++) printf("%4.6f ", shroud_out[i]); 
    }
	
    /* transform body force to wind frame    */
    /* FB, by looking at the data & physical intuition:
     * shroud_out[0] = Fxbody; shroud_out[1] = -Fzbody      */
    shroud_out[1] = - shroud_out[1];
    shroudLDM[DRAG]  =  - ( shroud_out[0]*cos(alpharad) + shroud_out[1]*sin(alpharad) );
    shroudLDM[LIFT]  =  shroud_out[0]*sin(alpharad) - shroud_out[1]*cos(alpharad);
    /* fix unit from [N mm] to [N m] */
    shroudLDM[MOMENT] = shroud_out[2] * 0.001;  

    if (aero_verbose > 0) {
	printf(" + shroudLDM:  ");
	for(i=0; i<3; i++) printf("%4.6f ", shroudLDM[i]);
	printf("\n");
    }
    
    return 0;
}





