/*
 * surfalloc.c - S-Function for surface allocation
 *
 * MvN 4/27/96
 * FB, 5/19/96 -- modified with my notation
 *
 * Inputs:	Fxbody, Fzbody, elevator [N N rad]  
 *                 and state [x,z,theta,xdot,zdot,thetadot]
 * Outputs:	voltage[volt], paddle [rad], elevator[rad]
 * Inputargs:   none
 */

#define VERBOSE 0			/* verbosity level */
#define S_FUNCTION_NAME surfalloc
#include "simstruc.h"
#include <math.h>

#include "aero.h"

/* mdlInitializeSizes - initialize the sizes array */
static void mdlInitializeSizes(S)
SimStruct *S;
{
    ssSetNumContStates(    S, 0);      /* number of continuous states */
    ssSetNumDiscStates(    S, 0);      /* number of discrete states */
    ssSetNumInputs(        S, 9);      /* number of inputs: [Fzbody, Fxbody, delta_e], states */
    ssSetNumOutputs(       S, 3);      /* number of outputs */
    ssSetDirectFeedThrough(S, 1);      /* direct feedthrough flag */
    ssSetNumSampleTimes(   S, 1);      /* number of sample times */
    ssSetNumInputArgs(     S, 0);      /* number of input arguments (parms) */
    ssSetNumRWork(         S, 0);      /* number real work vector elements */
    ssSetNumIWork(         S, 0);      /* number int work vector elements */
    ssSetNumPWork(         S, 0);      /* number ptr work vector elements */

}

/* mdlInitializeSampleTimes - initialize the sample times array */
static void mdlInitializeSampleTimes(S)
SimStruct *S;
{
    static int firsttime = 1;
    /* Set things up to run as continuous time block */
    ssSetSampleTimeEvent(S, 0, 0.0);
    ssSetOffsetTimeEvent(S, 0, 0.0);


    if(firsttime == 0) return; 
    firsttime = 0;

    init_surfalloc(0);
    aero_verbose = VERBOSE;
}

/* mdlInitializeConditions - initialize the states */
static void mdlInitializeConditions(x0, S)
double *x0;
SimStruct *S;
{
    static int firsttime = 1;

    if(firsttime == 1) {
	firsttime = 0;
    }
}

/* mdlOutputs - compute the outputs */
static void mdlOutputs(y, x, u, S, tid)
double *y, *x, *u;
SimStruct *S;
int tid;
{
  /* u is input to block: 
   * components =[Fxbody, Fzbody, delta_e] and [full 6 states]
   * every angle is in radians
   */
  double alphaVgamma[3], thetarad = u[5], xdot = u[6], zdot = u[7],
         Fbody[2], Vm_deltap[2];
  
  /* compute angle_of_attack, Vel, gamma */
  compute_alphaVgamma(thetarad, xdot, zdot, alphaVgamma);


  /* compute inputs to shroud with inverse table: */
  Fbody[0] = u[0];
  Fbody[1] = u[1];
  force_to_voltflap(alphaVgamma, Fbody, Vm_deltap);
  y[0] = Vm_deltap[0];  
  y[1] = Vm_deltap[1];  
  
  /* pass though the elevator: */
  y[2] = u[2];    
}

/* mdlUpdate - perform action at major integration time step (not used) */
static void mdlUpdate(x, u, S, tid)
double *x, *u;
SimStruct *S;
int tid;
{
    /* Not used for this model */
}

/* mdlDerivatives - compute the derivatives */
static void mdlDerivatives(dx, x, u, S, tid)
double *dx, *x, *u;
SimStruct *S;
int tid;
{
/* not used */
}

/* mdlTerminate - called when the simulation is terminated */
static void mdlTerminate(S)
    SimStruct *S;
{
}

/* Trailer information to set everything up for simulink usage */
#ifdef MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#endif
