/* 
 * wing.c - aerodynamic computations for wing
 *
 * MvN 3/24/96.  
 *
 */

#include "aero.h"

/* Global variables */
LU_DATA *wing_table = NULL;

int init_wing(char *table)
{
    char filename[1024];
    double tmp0, tmp1;

    if (table == NULL) table = WING_LOOKUP_TABLE;
    if (dfspec(table, filename) == NULL ||
	(wing_table = lu_load(filename)) == NULL) {
	printf("\n  Error Loading %s !\n", table);
	return -1;
    }
    if(aero_verbose>0) {
	printf("Loaded %s\n", table);
	lu_dispinfo(wing_table);
    }
    
    return 1;
    
}

/************************************************************/
int compute_wingLDM(double Vel, double alpharad, double delta_e_rad, 
		    double *wingLDM)
/* 
 * Wing forces in LDM.
 * Drag is positive when wing-force is against motion. (hence Drag>0 for wing)
 * Lift is always up. 
 * Moment is about  c.m. of whole fan. This depends on relative position 
 * of wing on shroud.
 * Lift and drag  for planar fan: scaling has to be done in spatial 
 * frame. 
 */
/************************************************************/
{
    double dynpress = 0.5*RHO*Vel*Vel, wing_in[2], wing_coef[3], 
           wingspan = WING_OUTSIDE_DIST - WING_INSIDE_DIST;
    int i;
    
    wing_in[0] = alpharad*RAD2DEG; 
    wing_in[1] = delta_e_rad*RAD2DEG; 
    lu_lookup(wing_table, wing_in, wing_coef);
    
    /* planar fan: */
    wingLDM[LIFT]   = dynpress*wing_coef[LIFT]*SPAN*CHORD;
    wingLDM[DRAG]   = dynpress*wing_coef[DRAG]*SPAN*CHORD;
    wingLDM[MOMENT] = dynpress*wing_coef[MOMENT]*SPAN*CHORD*CHORD;
    
    /* 
     * Account for moment of lift and drag of wing about center of mass
     * of total. The wing was mounted 0.137 m from the leading edge 
     * in the windtunnel. The c.m. is 0.06 m from the l.e.. 
     * Mcm = Mexp -(L*cos(alpha) + D*sin(alpha))*(d(le, sensor) - d(le, cm))
     */
    
    wingLDM[MOMENT] += (WING_SENSOR_DIST - LE_CM_DIST)*(-cos(alpharad)*wingLDM[LIFT] - sin(alpharad)*wingLDM[DRAG]);
    
    if(aero_verbose>0) {
/*	printf("alpha [deg], delta_e [deg], Vel: %lf %lf %lf ", alpharad * RAD2DEG, delta_e_rad * RAD2DEG, Vel);  */
	printf(" + wingLDM:  ");
	for(i=0; i<3; i++) printf("%lf ", wingLDM[i]);
	printf("\n");
    }  
    return 3;
} /*** end compute_wingLDM() ***/





